local BarHeight=18
local BarWidth=200
local TotalBuffs=5
local PlayersShown=8
local BuffSpacing=18

local BuffWindow_Functions={}

local RL = AceLibrary("Roster-2.1")
local aura = AceLibrary("SpecialEvents-Aura-2.0")

function Sort_RaidBuffs(a,b)
	if a.totalBuffs<b.totalBuffs then
		return true
	elseif a.totalBuffs>b.totalBuffs then
		return false
	elseif a.name<b.name then
		return true
	end
	return false
end

function Sort_PallyBuffs(a,b)
	if a.class<b.class then
		return true
	elseif a.class>b.class then
		return false
	elseif a.name<b.name then
		return true
	end
	return false
end



BigBrother_BuffTable={
	{
		name="Raid Buffs",
		sortFunc=Sort_RaidBuffs,
		buffs={
			{{"Arcane Intellect","Interface\\Icons\\Spell_Holy_MagicalSentry.blp"},{"Arcane Brilliance","Interface\\Icons\\Spell_Holy_ArcaneIntellect.blp"}},
			{{"Power Word: Fortitude","Interface\\Icons\\Spell_Holy_WordFortitude.blp"},{"Prayer of Fortitude","Interface\\Icons\\Spell_Holy_PrayerOfFortitude.blp"}},
			{{"Mark of the Wild","Interface\\Icons\\Spell_Nature_Regeneration.blp"},{"Gift of the Wild","Interface\\Icons\\Spell_Nature_Regeneration.blp"}},
			{{"Shadow Protection","Interface\\Icons\\Spell_Shadow_AntiShadow.blp"},{"Prayer of Shadow Protection","Interface\\Icons\\Spell_Holy_PrayerofShadowProtection.blp"}},
			{{"Divine Spirit","Interface\\Icons\\Spell_Holy_DivineSpirit"},{"Prayer of Spirit","Interface\\Icons\\Spell_Holy_PrayerofSpirit.blp"}}
		}
	},
	{
		name="Paladin Buffs",
		sortFunc=Sort_PallyBuffs,
		buffs={
			{{"Blessing of Kings","Interface\\Icons\\Spell_Magic_MageArmor.blp"},{"Greater Blessing of Kings","Interface\\Icons\\Spell_Magic_GreaterBlessingofKings.blp"}},
			{{"Blessing of Light","Interface\\Icons\\Spell_Holy_PrayerOfHealing02.blp"},{"Greater Blessing of Light","Interface\\Icons\\Spell_Holy_GreaterBlessingofLight.blp"}},
			{{"Blessing of Might","Interface\\Icons\\Spell_Holy_FistOfJustice.blp"},{"Greater Blessing of Might","Interface\\Icons\\Spell_Holy_GreaterBlessingofKings.blp"}},
			{{"Blessing of Salvation","Interface\\Icons\\Spell_Holy_SealOfSalvation.blp"},{"Greater Blessing of Salvation","Interface\\Icons\\Spell_Holy_GreaterBlessingofSalvation.blp"}},
			{{"Blessing of Wisdom","Interface\\Icons\\Spell_Holy_SealOfWisdom.blp"},{"Greater Blessing of Wisdom","Interface\\Icons\\Spell_Holy_GreaterBlessingofWisdom.blp"}}
		}
	},
	{
		name="Melee Elixirs",
		sortFunc=Sort_PallyBuffs,
		filter={ROGUE=true,HUNTER=true,SHAMAN=true,DRUID=true,WARRIOR=true},
		buffs={
			{{"Flask of Relentless Assault","Interface\\Icons\\INV_Potion_117.blp"}},
			{{"Fel Strength Elixir","Interface\\Icons\\INV_Potion_149.blp"},{"Onslaught Elixir","Interface\\Icons\\INV_Potion_58.blp"}},
			{{"Major Agility","Interface\\Icons\\INV_Potion_127.blp","Elixir of Major Agility"}},
			{{"Major Strength","Interface\\Icons\\INV_Potion_147.blp","Elixir of Major Strength"}},
			{{"Elixir of Mastery","Interface\\Icons\\INV_Potion_111.blp"}}
		}
	},
	{
		name="Caster Elixirs",
		sortFunc=Sort_PallyBuffs,
		filter={MAGE=true,PRIEST=true,SHAMAN=true,DRUID=true,WARLOCK=true},
		buffs={
			{},
			{{"Supreme Power","Interface\\Icons\\INV_Potion_41.blp","Flask of Supreme Power"}},
			{{"Adept's Elixir","Interface\\Icons\\INV_Potion_96.blp"},{"Arcane Elixir","Interface\\Icons\\INV_Potion_30.blp"},{"Greater Arcane Elixir","Interface\\Icons\\INV_Potion_25.blp"}},
			{{"Major Firepower","Interface\\Icons\\INV_Potion_146.blp","Elixir of Major Firepower"},{"Major Shadow Power","Interface\\Icons\\INV_Potion_145.blp","Elixir of Major Shadow Power"},{"Major Frost Power","Interface\\Icons\\INV_Potion_148.blp","Elixir of Major Frost Power"}},
			{{"Greater Mana Regeneration","Interface\\Icons\\INV_Potion_151.blp","Elixir of Major Mageblood"},{"Mana Regeneration","Interface\\Icons\\INV_Potion_45.blp","Mageblood Potion"}},
			{{"Elixir of Mastery","Interface\\Icons\\INV_Potion_111.blp"}}
		}
	},
	{
		name="Healing Elixirs",
		sortFunc=Sort_PallyBuffs,
		filter={PALADIN=true,PRIEST=true,SHAMAN=true,DRUID=true},
		buffs={
			{},
			{{"Flask of Mighty Restoration","Interface\\Icons\\INV_Potion_118.blp"}},
			{{"Adept's Elixir","Interface\\Icons\\INV_Potion_96.blp"},{"Healing Power","Interface\\Icons\\INV_Potion_142.blp"}},
			{{"Greater Mana Regeneration","Interface\\Icons\\INV_Potion_151.blp","Elixir of Major Mageblood"},{"Mana Regeneration","Interface\\Icons\\INV_Potion_45.blp","Mageblood Potion"}},
			{{"Elixir of Mastery","Interface\\Icons\\INV_Potion_111.blp"}}
		}
	},
	{
		name="Tanking Elixirs",
		sortFunc=Sort_PallyBuffs,
		filter={PALADIN=true,WARRIOR=true,DRUID=true},
		buffs={
			{},
			{{"Flask of Fortification","Interface\\Icons\\INV_Potion_119.blp"},{"Flask of the Titans","Interface\\Icons\\INV_Potion_62.blp"}},
			{{"Major Agility","Interface\\Icons\\INV_Potion_127.blp","Elixir of Major Agility"}},
			{{"Major Armor","Interface\\Icons\\INV_Potion_122.blp","Elixir of Major Defense"},{"Greater Armor","Interface\\Icons\\INV_Potion_66.blp","Elixir of Superior Defense"}},
			{{"Elixir of Mastery","Interface\\Icons\\INV_Potion_111.blp"}}
		}
	}
}



function BuffWindow_Functions:CreateBuffRow(parent, xoffset, yoffset)
	local Row=CreateFrame("FRAME",nil,parent)

	Row:SetPoint("TOPLEFT",parent,"TOPLEFT",xoffset,yoffset)
	Row:SetHeight(BarHeight)
	Row:SetWidth(BarWidth)
	Row:Show()
	
	Row.Background=Row:CreateTexture(nil,"BACKGROUND")
	Row.Background:SetAllPoints(Row)
	Row.Background:SetTexture("Interface\\Buttons\\WHITE8X8.blp")
	Row.Background:SetGradientAlpha("HORIZONTAL",1.0/2,0.0,0.0,0.8,1.0/2,0.0,0.0,0.0)
	Row.Background:Show()

	Row.Name=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")	
	Row.Name:SetPoint("LEFT",Row,"LEFT",4,0)
	Row.Name:SetTextColor(1.0,1.0,1.0)
	Row.Name:SetText("Test")

	

	Row.Buff={}
	for i=1,TotalBuffs do
		Row.Buff[i]=CreateFrame("FRAME",nil,Row)		
		Row.Buff[i]:SetPoint("RIGHT",Row,"RIGHT",-4-(TotalBuffs-i)*BuffSpacing,0)
		Row.Buff[i]:SetHeight(16)
		Row.Buff[i]:SetWidth(16)

		Row.Buff[i].texture=Row.Buff[i]:CreateTexture(nil,"OVERLAY")
		Row.Buff[i].texture:SetAllPoints(Row.Buff[i])
		Row.Buff[i].texture:SetTexture("Interface\\Buttons\\UI-CheckBox-Check.blp")
			
		Row.Buff[i].BuffName="None"
		--GameTooltip:ClearLines();GameTooltip:AddLine(this.BuffName);GameTooltip:Show()
		Row.Buff[i]:SetScript("OnEnter", BuffWindow_Functions.OnEnterBuff)
		Row.Buff[i]:SetScript("OnLeave", BuffWindow_Functions.OnLeaveBuff)
		Row.Buff[i]:EnableMouse()

		Row.Buff[i]:Show()
	end

	Row.SetPlayer=BuffWindow_Functions.SetPlayer
	Row.SetBuffValue=BuffWindow_Functions.SetBuffValue
	Row.SetBuffIcon=BuffWindow_Functions.SetBuffIcon
	Row.SetBuffText=BuffWindow_Functions.SetBuffText

	return Row
end

function BuffWindow_Functions:OnEnterBuff()
	GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT")
	GameTooltip:SetText(self.BuffName, 1, 1, 1, 1)
	GameTooltip:Show()
end

function BuffWindow_Functions:OnLeaveBuff()
	GameTooltip:Hide()
end


function BuffWindow_Functions:SetPlayer(player,class)
	self.Name:SetText(player)
	local color=RAID_CLASS_COLORS[class]
	self.Background:SetGradientAlpha("HORIZONTAL",color.r/1.5,color.g/1.5,color.b/1.5,0.8,color.r/1.5,color.g/2,color.b/1.5,0)
end

function BuffWindow_Functions:SetBuffValue(num,enabled)
	if enabled then
		self.Buff[num]:Show()
	else
		self.Buff[num]:Hide()
	end
end

function BuffWindow_Functions:SetBuffIcon(num,texture)
	self.Buff[num].texture:SetTexture(texture)
end

function BuffWindow_Functions:SetBuffText(num,text)
	self.Buff[num].BuffName=text
end




function CreateBuffWindow()
	local BuffWindow

	if BigBrother_BuffWindow then
		BuffWindow_UpdateBuffs()
		BuffWindow_UpdateWindow()
		BigBrother_BuffWindow:Show()
		return
	end

	BuffWindow=CreateFrame("FRAME",nil,UIParent)


	BuffWindow:ClearAllPoints()
	BuffWindow:SetPoint("CENTER",UIParent)

	BuffWindow:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            insets = { left = 4, right = 4, top = 4, bottom = 4 }});
	BuffWindow:SetBackdropColor(0,0,0,0.5);

	BuffWindow:SetWidth(BarWidth+16+24);
	BuffWindow:SetHeight(190);

	BuffWindow:SetScript("OnMouseDown", function() 
						if ( ( ( not BigBrother_BuffWindow.isLocked ) or ( BigBrother_BuffWindow.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
						  BigBrother_BuffWindow:StartMoving();
						  BigBrother_BuffWindow.isMoving = true;
						 end
						end)
	BuffWindow:SetScript("OnMouseUp", function() 
						if ( BigBrother_BuffWindow.isMoving ) then
						  BigBrother_BuffWindow:StopMovingOrSizing();
						  BigBrother_BuffWindow.isMoving = false;
						 end
						end)
					
	BuffWindow:SetScript("OnHide", function() 
						if ( BigBrother_BuffWindow.isMoving ) then
						  BigBrother_BuffWindow:StopMovingOrSizing();
						  BigBrother_BuffWindow.isMoving = false;
						 end
						end)

	BuffWindow:SetMovable(true)
	BuffWindow:EnableMouse()
	BuffWindow:Show()

	BuffWindow.Title=BuffWindow:CreateFontString(nil,"OVERLAY","GameFontNormal")
	BuffWindow.Title:SetPoint("TOP",BuffWindow,"TOP",0,-8)
	BuffWindow.Title:SetTextColor(1.0,1.0,1.0)

	BuffWindow.LeftButton=CreateFrame("Button",nil,BuffWindow)
	BuffWindow.LeftButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up.blp")
	BuffWindow.LeftButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down.blp")
	BuffWindow.LeftButton:SetWidth(16)
	BuffWindow.LeftButton:SetHeight(18)
	BuffWindow.LeftButton:SetPoint("TOPLEFT",BuffWindow,"TOPLEFT",64,-5)
	BuffWindow.LeftButton:SetScript("OnClick",function() 
							BigBrother_BuffWindow.SelectedBuffs=BigBrother_BuffWindow.SelectedBuffs-1
							if BigBrother_BuffWindow.SelectedBuffs==0 then
								BigBrother_BuffWindow.SelectedBuffs=table.getn(BigBrother_BuffTable)
							end
							BuffWindow_UpdateBuffs()
							BuffWindow_UpdateWindow()
						end)

	BuffWindow.RightButton=CreateFrame("Button",nil,BuffWindow)
	BuffWindow.RightButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up.blp")
	BuffWindow.RightButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down.blp")
	
	BuffWindow.RightButton:SetWidth(16)
	BuffWindow.RightButton:SetHeight(18)
	BuffWindow.RightButton:SetPoint("TOPRIGHT",BuffWindow,"TOPRIGHT",-64,-5)
	BuffWindow.RightButton:SetScript("OnClick",function() 
							BigBrother_BuffWindow.SelectedBuffs=BigBrother_BuffWindow.SelectedBuffs+1
							if BigBrother_BuffWindow.SelectedBuffs>table.getn(BigBrother_BuffTable) then
								BigBrother_BuffWindow.SelectedBuffs=1
								
							end
							BuffWindow_UpdateBuffs()
							BuffWindow_UpdateWindow()
						end)
	
	BuffWindow.CloseButton=CreateFrame("Button",nil,BuffWindow)
	BuffWindow.CloseButton:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up.blp")
	BuffWindow.CloseButton:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down.blp")
	BuffWindow.CloseButton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight.blp")
	BuffWindow.CloseButton:SetWidth(20)
	BuffWindow.CloseButton:SetHeight(20)
	BuffWindow.CloseButton:SetPoint("TOPRIGHT",BuffWindow,"TOPRIGHT",-4,-4)
	BuffWindow.CloseButton:SetScript("OnClick",function() BigBrother_BuffWindow:Hide() end)


	BuffWindow.Rows={}
	for i=1,8 do
		BuffWindow.Rows[i]=BuffWindow_Functions:CreateBuffRow(BuffWindow,8,-4-i*(BuffSpacing+2))
	end

	BuffWindow.ScrollBar=CreateFrame("SCROLLFRAME","BuffWindow_ScrollBar",BuffWindow,"FauxScrollFrameTemplate")
	BuffWindow.ScrollBar:SetScript("OnVerticalScroll", function() FauxScrollFrame_OnVerticalScroll(20, BuffWindow_UpdateWindow) end)

	BuffWindow.ScrollBar:SetPoint("TOPLEFT", BuffWindow.Rows[1], "TOPLEFT", 0, 0)
	BuffWindow.ScrollBar:SetPoint("BOTTOMRIGHT", BuffWindow.Rows[8], "BOTTOMRIGHT", 0, 0)
    

	BigBrother_BuffWindow=BuffWindow
	BigBrother_BuffWindow.SelectedBuffs=1
	BuffWindow_UpdateBuffs()
	BuffWindow_UpdateWindow()
end

function BuffWindow_UpdateBuffs()
	local unit
	local PlayerList={}
	local BuffChecking=BigBrother_BuffTable[BigBrother_BuffWindow.SelectedBuffs]
	local Filter=BuffChecking.filter

	for unit in RL:IterateRoster(false) do
		if (not Filter) or Filter[unit.class] then
			local player={}
			player.name=unit.name
			player.class=unit.class
			player.totalBuffs=0
			player.buff={}
			for i, BuffList in pairs(BuffChecking.buffs) do
				for _, buffs in pairs(BuffList) do
					if aura:UnitHasBuff(unit.unitid, buffs[1]) then
						player.totalBuffs=player.totalBuffs+1
						player.buff[i]=buffs
						break
					end
				end
			end
			table.insert(PlayerList,player)
		end
	end

	table.sort(PlayerList,BuffChecking.sortFunc)
	BigBrother_BuffWindow.List=PlayerList
end

function BuffWindow_UpdateWindow()
	local PlayerList=BigBrother_BuffWindow.List
	local Rows=BigBrother_BuffWindow.Rows
	
	FauxScrollFrame_Update(BigBrother_BuffWindow.ScrollBar, table.getn(PlayerList), PlayersShown, 20)
	local offset = FauxScrollFrame_GetOffset(BigBrother_BuffWindow.ScrollBar)
	
	BigBrother_BuffWindow.Title:SetText(BigBrother_BuffTable[BigBrother_BuffWindow.SelectedBuffs].name)


	for i=1,PlayersShown do
		if PlayerList[i+offset] then
			local Player=PlayerList[i+offset]
			Rows[i]:SetPlayer(Player.name,Player.class)
			for j=1,5 do
				if Player.buff[j] then
					Rows[i]:SetBuffIcon(j,Player.buff[j][2])
					if Player.buff[j][3] then
						Rows[i]:SetBuffText(j,Player.buff[j][3])
					else
						Rows[i]:SetBuffText(j,Player.buff[j][1])
					end
					Rows[i]:SetBuffValue(j,true)
				else
					Rows[i]:SetBuffValue(j,false)
				end
			end
			Rows[i]:Show()
		else
			Rows[i]:Hide()
		end
	end
end